/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.language.markup.xsp;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
 * This helper class takes care of contstructing queries
 * and cursor positioning (paging) for all different kinds
 * of databases
 *
 * based on the orginal esql.xsl
 * @author <a href="mailto:tcurdt@dff.st">Torsten Curdt</a>
 * @version CVS $Id: EsqlQuery.java,v 1.11 2002/02/22 07:00:08 cziegeler Exp $
 */

public class EsqlQuery {
  private Connection connection = null;
  private Statement statement = null;
  private PreparedStatement preparedStatement = null;
  private ResultSet resultSet = null;
  private ResultSetMetaData resultSetMetaData = null;
  private boolean hasResultSet = false;
  private boolean resultSetValid = false;
  private int position = -1;
  private int maxRows = -1;
  private int skipRows = 0;
  private boolean keepgoing = true;
  private java.util.Hashtable groupingVars = new java.util.Hashtable();

  private String query;
  private int limitMethod;

  public EsqlQuery( EsqlConnection connection, String query ) {
    this.connection = connection;
    this.limitMethod = connection.getLimitMethod();
    this.query = query;
  }

  public EsqlQuery( ResultSet aResultSet ) {
    this.connection = null;
    this.statement = null;
    this.query = null;
    this.resultSetValid = true;
    this.resultSet = aResultSet;
    this.hasResultSet = (this.resultSet != null);
  }

  public int getSkipRows() {
    return(skipRows);
  }

  public void setSkipRows( int i ) {
    this.skipRows = i;
  }

  public int getMaxRows() {
    return(maxRows);
  }

  public void setMaxRows( int i ) {
    this.maxRows = i;
  }

  public int getCurrentRow() {
    return(position);
  }

  public String getQueryString() {
    switch(limitMethod) {
      case EsqlConnection.LIMIT_METHOD_POSTGRESQL:
        if (skipRows > 0) {
          if (maxRows > -1) {
            return(new StringBuffer(query).append(" LIMIT ").append(maxRows).append(",").append(skipRows).toString());
          }
          else {
            return(new StringBuffer(query).append(" OFFSET ").append(skipRows).toString());
          }
        }
        else {
          if (maxRows > -1) {
            return(new StringBuffer(query).append(" LIMIT ").append(maxRows).toString());
          }
          else {
            return(query);
          }
        }
      case EsqlConnection.LIMIT_METHOD_MYSQL:
        if (skipRows > 0) {
          if (maxRows > -1) {
            return(new StringBuffer(query).append(" LIMIT ").append(skipRows).append(",").append(maxRows).toString());
          }
          else {
            throw new RuntimeException("MySQL does not support a skip of rows only");
          }
        }
        else {
          if (maxRows > -1) {
            return(new StringBuffer(query).append(" LIMIT ").append(maxRows).toString());
          }
          else {
            return(query);
          }
        }
      default:
        return(query);
    }
  }

  public PreparedStatement prepareStatement() throws SQLException {
    switch(limitMethod) {
      case EsqlConnection.LIMIT_METHOD_JDBC:
        preparedStatement = connection.prepareStatement( getQueryString(), ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        break;
      default:
        preparedStatement = connection.prepareStatement( getQueryString() );
    };
    statement = preparedStatement;
    return(preparedStatement);
  }


  public CallableStatement prepareCall() throws SQLException {
    switch(limitMethod) {
      case EsqlConnection.LIMIT_METHOD_JDBC:
        preparedStatement = connection.prepareCall( getQueryString(), ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        break;
      default:
        preparedStatement = connection.prepareCall( getQueryString() );
    };
    statement = preparedStatement;
    return((CallableStatement)preparedStatement);
  }

  public PreparedStatement getPreparedStatement() {
    return(preparedStatement);
  }

  public CallableStatement getCallableStatement() {
    return((CallableStatement) preparedStatement);
  }

  public ResultSet getResultSet() {
    return(resultSet);
  }

  public boolean nextRow() throws SQLException {
    position++;
    return(resultSet.next());
  }

  public boolean keepGoing() {
    return(keepgoing);
  }

  public void setKeepGoing( boolean still ) {
    keepgoing = still;
  }

  public Object setGroupingVar( String key, Object value) {
    return groupingVars.put(key,value);
  }

  public Object getGroupingVar( String key) {
    return groupingVars.get(key);
  }

  public ResultSetMetaData getResultSetMetaData() {
    return(resultSetMetaData);
  }

  public Statement createStatement() throws SQLException {
    switch(limitMethod) {
      case EsqlConnection.LIMIT_METHOD_JDBC:
        statement = connection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        break;
      default:
        statement = connection.createStatement();
    };
    return(statement);
  }

  public Statement getStatement() {
    return(statement);
  }

  public boolean getMoreResults() throws SQLException {
    return (statement!=null? statement.getMoreResults() : false);
  }

  public boolean execute() throws SQLException {
      return this.execute(false);
  }

  public boolean execute( int resultSetFromObject ) throws SQLException {
      hasResultSet = this.execute(false);
      resultSet = (ResultSet) ((CallableStatement) preparedStatement).getObject(1);
      resultSetValid = true;
      return hasResultSet;
  }

  /**
   * some brain dead DBMSs (Informix) don't like their callable
   * statements to be 'execute'd but require 'executeQuery' instead.
   */
  public boolean execute( boolean needsQuery ) throws SQLException {
      if ( needsQuery ) {
          if (preparedStatement != null) {
              resultSet = preparedStatement.executeQuery();
          }
          else {
              resultSet = statement.executeQuery( getQueryString() );
          }
          hasResultSet = (resultSet != null);
          resultSetValid = true;
      } else {
          if (preparedStatement != null) {
              hasResultSet = preparedStatement.execute();
          }
          else {
              hasResultSet = statement.execute( getQueryString() );
          }
          resultSetValid = false;
      }
      return(hasResultSet);
  }

  public boolean hasResultSet() {
    return(this.hasResultSet);
  }

  public int rowCount() throws SQLException {
    switch(limitMethod) {
      case EsqlConnection.LIMIT_METHOD_JDBC:
          // TC: I'm not quite sure if synchronized is really necessary since
          // the XSP page will be poolable
          synchronized (resultSet) {
            int currentRow = resultSet.getRow();
            resultSet.last();
            int count = resultSet.getRow();
            if (currentRow > 0) {
              resultSet.absolute(currentRow);
            }
            else {
              resultSet.first();
             resultSet.relative(-1);
            }
            return(count);
          }
      default:
        // select count(*)
        throw new RuntimeException("not yet implemented");
    }
  }

  public void getResultRows() throws SQLException {
    if ( !resultSetValid ) {
        resultSet = statement.getResultSet();
    }
    resultSetMetaData = resultSet.getMetaData();

    if (skipRows > 0) {
      switch(limitMethod) {
        case EsqlConnection.LIMIT_METHOD_POSTGRESQL:
          // in clause
          position = skipRows;
          break;
        case EsqlConnection.LIMIT_METHOD_MYSQL:
          // in clause
          position = skipRows;
          break;
        case EsqlConnection.LIMIT_METHOD_JDBC:
          resultSet.absolute(skipRows);
          position = skipRows;
          break;
        default:
          while (resultSet.next()) {
            position++;
            if (position == skipRows) {
               break;
            }
          }
      };
    }
  }
}
