;; functions.scm
;;
;; Support for functions with named arguments.
;;
;; Author: Ovidiu Predescu <ovidiu@cup.hp.com>
;; Date: January 24, 2002
;;

(define-syntax function
  (lambda (x)
    (syntax-case x ()
     ((_ (name sitemap env arg ...) body ...)
      (syntax
       (define (name sitemap env params)
	 (let ((arg (extract-param params 'arg))
	       ...)
	   body ...)))))))

(define (extract-param params arg)
  (if (pair? params)
      (let ((r (assoc (symbol->string arg) params)))
	(if (pair? r)
	    (cdr r)
	    '()))
      '()))

(define (function-call func sitemap env args)
  (let ((params (cdr (assq 'params args))))
    (apply func sitemap env (list params))))

(define (function-call-v sitemap env args)
  (let ((funname (cdr (assq 'function args)))
	(params (cdr (assq 'params args))))
    (eval `(,(string->symbol funname) ,sitemap ,env (quote ,params))
	  (interaction-environment))))

(define (resource-call-v sitemap env args)
  (let ((funname (name->resource (cdr (assq 'resource args))))
	(params (cdr (assq 'params args))))
    (eval `(,funname ,sitemap ,env (quote ,params))
	  (interaction-environment))))

(define (send-page sitemap env resource-name params bean-dict)
  (call/cc
   (lambda (k)
     (let ((kont-id (continuations:register k))
	   (suicide (environ:get-attr env "suicide")))
       (environ:set-attr env "kont" kont-id)
       (environ:set-attr env "bean-dict" bean-dict)
       (printf "send-page: kont ~s, kont id ~s~%" k kont-id)
       (resource-call-v sitemap env
			(list (cons 'resource resource-name)
			      (cons 'params params)))
       (suicide k))
     )))

(function (schecoon:handle-continuation sitemap env kont-id)
  (printf "handle-continuation invoked ~s~%" kont-id)
  (let ((kont (continuations:get kont-id)))
    (if kont (kont kont))))


;(function-call-v 1 2 '((function . "my-function")
;		       (params . (("a" . "3") ("b" . "4")))))

;(function-call my-function 1 2 '((params . (("a" . "3") ("b" . "4")))))

;(my-function 's 'e '(("a" . "1") ("b" . "2")))

;(my-function 's 'e (list (cons 'params
;			       (list (cons "source" "a")))
;			 (cons 'resource "my-function")))
