package org.apache.cocoon.components.treeprocessor.sitemap;

import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.ComponentSelector;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.cocoon.Constants;
import org.apache.cocoon.components.flow.Interpreter;
import org.apache.cocoon.components.flow.AbstractInterpreter;
import org.apache.cocoon.components.treeprocessor.AbstractProcessingNode;
import org.apache.cocoon.components.treeprocessor.InvokeContext;
import org.apache.cocoon.environment.Context;
import org.apache.cocoon.environment.Environment;

public class ScriptNode extends AbstractProcessingNode
  implements Composable, Contextualizable
{
  ComponentManager manager;
  String source;
  String language;
  Context context;

  public ScriptNode(String source, String language)
  {
    this.source = source;
    this.language = language;
  }
  
  /**
   * This method should never be called by the TreeProcessor, since a
   * <map:script> element should not be in an "executable" sitemap
   * node.
   *
   * @param env an <code>Environment</code> value
   * @param context an <code>InvokeContext</code> value
   * @return a <code>boolean</code> value
   * @exception Exception if an error occurs
   */
  public boolean invoke(Environment env, InvokeContext context)
    throws Exception
  {
    return true;
  }

  public void contextualize(org.apache.avalon.framework.context.Context context)
    throws ContextException
  {
    this.context = (Context)context.get(Constants.CONTEXT_ENVIRONMENT_CONTEXT);
  }

  /**
   *
   * Load the script specified by this node.
   *
   * @param manager a <code>ComponentManager</code> value
   */
  public void compose(ComponentManager manager)
    throws ComponentException
  {
    this.manager = manager;

    try {
      ComponentSelector selector
        = (ComponentSelector)manager.lookup(Interpreter.ROLE);
      // Obtain the Interpreter instance for this language
      Interpreter interpreter = (Interpreter)selector.select(language);
      if (interpreter instanceof AbstractInterpreter)
        ((AbstractInterpreter)interpreter).register(source);
      // else FIXME: what to do if interpreter doesn't inherit from
      // AbstractInterpreter
    }
    catch (Exception ex) {
      ex.printStackTrace();
      System.out.println("ScriptNode: Couldn't read the source file "
                         + source + " in language " + language);
      throw new ComponentException("ScriptNode: Couldn't read the source file "
                                   + source + " in language " + language);
    }
  }
}
