/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.sitemap;

import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.Loggable;
import org.apache.cocoon.components.source.CocoonSourceFactory;

/**
 * The Scheme sitemap handler. Cocoon's SitemapManager class needs to
 * be configured to use this handler instead of the Cocoon's default
 * one, which compiles the XML sitemap to Java code.
 *
 * This handler uses an instance of {@link SchemeSitemap} instead.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since December 18, 2001
 */
public class SchemeSitemapHandler
  extends org.apache.cocoon.sitemap.Handler
  implements Contextualizable, Composable, Configurable
{
  Context context;
  Configuration confs;

  public void contextualize(Context context)
    throws ContextException
  {
    this.context = context;
  }

  /**
   * Obtain the ComponentManager instance and wrap it into a Scheme
   * object, that's passed later to the Scheme sitemap function.
   *
   * @param manager a <code>ComponentManager</code> value
   */
  public void compose(ComponentManager manager)
    throws ComponentException
  {
    super.compose(manager);
  }

  /**
   * Properly setup the Scheme interpreter, by loading the heap file.
   *
   * @param sconf a <code>Configuration</code> value
   * @exception ConfigurationException if an error occurs
   */
  public void configure(Configuration confs)
    throws ConfigurationException
  {
    this.confs = confs;
  }

  public void run()
  {
    try {
      SchemeSitemap smap = new SchemeSitemap();
      ((Loggable)smap).setLogger(getLogger());
      ((Composable)smap).compose(manager);
      ((Contextualizable)smap).contextualize(context);
      ((Configurable)smap).configure(confs);
      ((SchemeSitemap)smap).parseSitemap(resolve(sourceFileName));
      // add the source factory for the cocoon protocol
      sourceHandler
        .addFactory("cocoon",
                    new CocoonSourceFactory(smap, smap.getComponentManager()));
      sitemap = smap;
    }
    catch (Exception ex) {
      ex.printStackTrace();
      getLogger().debug("Cannot create sitemap: " + ex);
    }
    finally {
      this.regeneration = null;
      this.isRegenerationRunning = false;
      this.contextSource.recycle();
      this.contextSource = null;
    }
  }
}
