/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.util;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.net.URLConnection;

/**
 * REPL (Read Eval Print Loop) program that sends commands to a Scheme
 * interpreter running in a servlet. Assumes the URL uses basic
 * authentication.
 *
 * The best use of this REPL program is from within Emacs, as it will
 * have nice command history facilities.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since December  9, 2001
 */
public class SchemeServletCmdLine
{
  BufferedReader in;
  String url;
  
  public static void main(String[] args)
  {
    String url = "http://localhost:8080/schecoon/eval";

    if (args.length != 0) {
      if (args[0].indexOf("-h") != -1) {
        System.out.println("usage: SchemeServletCmdLine URL");
        System.exit(1);
      }
      else
        url = args[0];
    }

    System.out.println("Connecting to " + url);

    new SchemeServletCmdLine(url).repl();
  }

  public SchemeServletCmdLine(String url)
  {
    Authenticator.setDefault(new CmdLineAuthenticator());
    in = new BufferedReader(new InputStreamReader(System.in));
    this.url = url;
  }

  public void repl()
  {
    while (true) {
      System.out.print("scheme> ");

      String evalString;
      try {
        evalString = in.readLine();
        if (evalString == null) {
          System.out.println("\nQuit.");
          return;
        }
      }
      catch (IOException ex) {
        System.out.println("Cannot read input line");
        continue;
      }

      try {
        URL schemeURL = new URL(url);
        HttpURLConnection conn = (HttpURLConnection)schemeURL.openConnection();
        conn.setDoOutput(true);
        conn.setDoInput(true);
        conn.setRequestMethod("POST");

        OutputStream out = conn.getOutputStream();
        Writer wout = new OutputStreamWriter(out, "UTF-8");
        wout.write(evalString);
        wout.flush();
        wout.close();
        
        InputStream is = conn.getInputStream();
        BufferedReader bin = new BufferedReader(new InputStreamReader(is));
        String line;
        while ((line = bin.readLine()) != null) {
          System.out.println(line);
        }
        is.close();

        conn.disconnect();
        
      } catch (MalformedURLException e) {
        System.out.println ("Invalid URL");
      } catch (IOException e) {
        System.out.println ("Error reading URL");
      }
    }
  }

  class CmdLineAuthenticator extends Authenticator
  {
    public CmdLineAuthenticator() {}

    protected PasswordAuthentication getPasswordAuthentication()
    {
      String username = null;
      String password = null;

      try {
        System.out.print("Username: ");
        username = in.readLine();
        System.out.print("Password: ");
        // FIXME: how to read the password securely? Java doesn't seem
        // to have a nice way to do it:
        // http://www.javaworld.com/javaworld/javaqa/2000-04/01-qa-0407-command.html
        password = in.readLine();
      }
      catch (IOException ex) {
        System.out.println("Caught exception " + ex);
        System.exit(1);
      }

      return new PasswordAuthentication(username, password.toCharArray());
    }
  }
}
