/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/

package org.apache.cocoon.precept.stores.dom.simple;


import org.apache.cocoon.precept.*;
import org.apache.cocoon.precept.Constraint;
import org.apache.cocoon.precept.stores.dom.simple.Node;
import org.apache.cocoon.precept.stores.AbstractInstance;

import java.util.*;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.ComponentException;

/*
 * @version: Mar 14, 2002
 * @author: Torsten Curdt <tcurdt@dff.st>
 */
public class InstanceImpl extends AbstractInstance {
  private HashMap index = new HashMap();
  private Preceptor preceptor;
  private ElementNode root;

  public void setPreceptor(Preceptor preceptor) {
    this.preceptor = preceptor;
    preceptor.buildInstance(this);
  }

  private Node createNode(String xpath) throws InvalidXPathSyntaxException {
    try {
      StringBuffer currentPath = new StringBuffer();
      StringTokenizer tok = new StringTokenizer(xpath, "/", false);
      Node currentParent = root;
      boolean first = true;
      while (tok.hasMoreTokens()) {
        String level = tok.nextToken();
        if (!first) {
          currentPath.append("/");
        }
        else {
          first = false;
        }
        if (level.endsWith("[1]")) {
          level = level.substring(0, level.length() - 3);
        }
        currentPath.append(level);
        Node node = (Node) index.get(currentPath.toString());
        if (node != null) {
          getLogger().debug("found node [" + String.valueOf(currentPath) + "] in index");
          currentParent = node;
        }
        else {
          if (currentParent != null) {
            if (level.startsWith("@")) {
              if (level.indexOf("[") >= 0 || level.indexOf("]") >= 0) {
                throw new InvalidXPathSyntaxException(level);
              }
              if (preceptor != null) {
                node = new AttributeNode(level.substring(1), preceptor.getConstraitsFor(currentPath.toString()));
              }
              else {
                node = new AttributeNode(level.substring(1), null);
              }
              getLogger().debug("creating attribute [" + String.valueOf(currentPath) + "]");
              ((ElementNode) currentParent).addAttribute(node);
              index.put(currentPath.toString(), node);
              return (node);
            }
            else {
              if (preceptor != null) {
                node = new ElementNode(level, preceptor.getConstraitsFor(currentPath.toString()));
              }
              else {
                node = new ElementNode(level, null);
              }
              getLogger().debug("creating node [" + String.valueOf(currentPath) + "]");
              ((ElementNode) currentParent).addChild(node);
              index.put(currentPath.toString(), node);
              index.put(currentPath.toString() + "[1]", node);
            }
          }
          else {
            getLogger().debug("creating root node [" + String.valueOf(currentPath) + "]");
            if (preceptor != null) {
              node = root = new ElementNode(level, preceptor.getConstraitsFor(currentPath.toString()));
            }
            else {
              node = root = new ElementNode(level, null);
            }
            index.put(currentPath.toString(), node);
            index.put(currentPath.toString() + "[1]", node);
          }
        }
        currentParent = node;
      }
      return (currentParent);
    }
    catch (NoSuchNodeException e) {
      getLogger().error("hm.. this should not happen!");
      return (null);
    }
  }

  public void setValue(String xpath, Object value) throws PreceptorViolationException, InvalidXPathSyntaxException {
    setValue(xpath, value, null);
  }

  public void setValue(String xpath, Object value, Context context) throws PreceptorViolationException, InvalidXPathSyntaxException {
    Node node = (Node) index.get(xpath);
    if (node != null) {
      node.setValue((String) value);
    }
    else {
      if (preceptor != null) {
        getLogger().debug("checking preceptor for [" + String.valueOf(xpath) + "]");
        if (preceptor.isValidNode(xpath)) {
          node = createNode(xpath);
          node.setValue((String) value);
        }
        else {
          throw new PreceptorViolationException("[" + String.valueOf(xpath) + "] is prohibited by preceptor");
        }
      }
      else {
        getLogger().debug("no preceptor");
        node = createNode(xpath);
        node.setValue((String) value);
      }
    }
  }

  public Object getValue(String xpath) throws InvalidXPathSyntaxException, NoSuchNodeException {
    Node node = (Node) index.get(xpath);
    if (node != null) {
      return (node.getValue());
    }
    else {
      throw new NoSuchNodeException(xpath);
    }
  }

  public List validate(Context context) throws InvalidXPathSyntaxException {
    ArrayList all = null;
    for (Iterator it = index.keySet().iterator(); it.hasNext();) {
      String xpath = (String) it.next();
      if (!xpath.endsWith("[1]")) {
        try {
          List result = validate(xpath, context);
          if (result != null) {
            getLogger().debug("constraint violations for [" + String.valueOf(xpath) + "]");
            if (all == null) all = new ArrayList();
            all.addAll(result);
          }
          else {
            getLogger().debug("[" + String.valueOf(xpath) + "] is valid");
          }
        }
        catch (NoSuchNodeException e) {
          getLogger().error("hm... this should not happen!");
        }
      }
    }
    return (all);
  }

  public List validate(String xpath, Context context) throws InvalidXPathSyntaxException, NoSuchNodeException {
    Node node = (Node) index.get(xpath);
    if (node != null) {
      ArrayList result = null;
      List constraints = node.getConstraints();
      if (constraints != null) {
        for (Iterator it = constraints.iterator(); it.hasNext();) {
          Constraint constraint = (Constraint) it.next();
          if (constraint.isSatisfiedBy(getValue(xpath), context )) {
            getLogger().debug("[" + String.valueOf(xpath) + "] constraint [" + String.valueOf(constraint) + "] is satisfied");
          }
          else {
            getLogger().debug("[" + String.valueOf(xpath) + "] constraint [" + String.valueOf(constraint) + "] FAILED!");
            if (result == null) result = new ArrayList();
            result.add(constraint);
          }
        }
      }
      return (result);
    }
    else {
      getLogger().error("could not find node [" + String.valueOf(xpath) + "]");
      throw new NoSuchNodeException(xpath);
    }
  }

  public Preceptor getPreceptor() {
    return (preceptor);
  }

  public long getLastModified() {
    //NYI
    return 0;
  }

  public void toSAX(ContentHandler handler, boolean constraints) throws SAXException {
    if (root != null) {
      root.toSAX(handler, root, constraints);
    }
  }

  public String toString() {
    if (root != null) {
      StringBuffer sb = new StringBuffer();
      root.toStringBuffer(sb, root, 0);
      return (sb.toString());
    }
    else {
      return ("");
    }
  }

}
