/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.sunshine.connector;

import java.io.IOException;
import java.io.Serializable;
import org.apache.avalon.excalibur.source.SourceParameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.Source;
import org.apache.cocoon.environment.SourceResolver;
import org.xml.sax.SAXException;

/**
 * This is a resource. It consists of a name and a type and possibly some
 * parameters.
 *
 * @author <a href="mailto:cziegeler@s-und-n.de">Carsten Ziegeler</a>
 * @version CVS $Id: Resource.java,v 1.3 2002/02/22 06:57:17 cziegeler Exp $
*/
public final class Resource
implements Serializable {

    /** The resource type, e.g. file */
    private int resourceType;

    /** The resource identifier, e.g. file name */
    private String resourceIdentifier;

    /** The resource parameters (or null) */
    private SourceParameters parameters;

    /**
     * Construct a new resource
     */
    public Resource(SourceResolver resolver,
                    String identifier)
    throws ProcessingException, SAXException, IOException {
        if ( identifier.startsWith("class://") ) {
            this.resourceType = ResourceConnector.RESOURCE_TYPE_CLASS;
            this.resourceIdentifier = identifier.substring("class://".length());
        } else if ( identifier.indexOf(":/") == -1) {
            // relative, resolve it first
            Source source = null;
            try {
                source = resolver.resolve(identifier);
                identifier = source.getSystemId();
                if ( identifier.startsWith("file:") ) {
                    this.resourceType = ResourceConnector.RESOURCE_TYPE_FILE;
                    this.resourceIdentifier = identifier.substring("file:".length());
                } else {
                    this.resourceType = ResourceConnector.RESOURCE_TYPE_URI;
                    this.resourceIdentifier = identifier;
                }
            } finally {
                source.recycle();
            }
        } else {
            // check for relative cocoon:/ protocol
            if (identifier.startsWith("cocoon:/")
                && identifier.charAt("cocoon:/".length()) != '/') {

                // FIXME (CZ)
                throw new ProcessingException("Relative cocoon: URIs currently not supported.");
            }
            if ( identifier.startsWith("file:") ) {
                this.resourceType = ResourceConnector.RESOURCE_TYPE_FILE;
                this.resourceIdentifier = identifier.substring("file:".length());
            } else {
                this.resourceType = ResourceConnector.RESOURCE_TYPE_URI;
                this.resourceIdentifier = identifier;
            }
        }
    }

    /**
     * Get the resource type
     */
    public int getResourceType() {
        return this.resourceType;
    }

    /**
     * Get the resource identifier
     */
    public String getResourceIdentifier() {
        return this.resourceIdentifier;
    }

    /**
     * Set some parameters
     */
    public void setResourceParameters(SourceParameters pars) {
        this.parameters = pars;
    }

    /**
     * Get the parameters
     */
    public SourceParameters getResourceParameters() {
        return this.parameters;
    }

    /**
     * PP for debugging
     */
    public String toString() {
        return "org.apache.cocoon.sunshine.connector.Resource: " +
               this.resourceIdentifier +
               " (" + this.resourceType + ")";
    }
}