/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.sunshine.sunrise.acting;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.avalon.excalibur.source.SourceParameters;
import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.acting.ComposerAction;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.sunshine.sunrise.Constants;
import org.apache.cocoon.sunshine.sunrise.SunRise;

/**
 *  This action logs the current user into a given handler
 *
 * @author <a href="mailto:cziegeler@s-und-n.de">Carsten Ziegeler</a>
 * @version CVS $Id: LoginAction.java,v 1.2 2002/02/22 06:57:18 cziegeler Exp $
*/
public final class LoginAction
extends ComposerAction
implements ThreadSafe {

    public Map act(Redirector redirector,
                   SourceResolver resolver,
                   Map objectModel,
                   String source,
                   Parameters par)
    throws Exception {
        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("BEGIN act resolver="+resolver+
                                   ", objectModel="+objectModel+
                                   ", source="+source+
                                   ", par="+par);
        }

        final String handlerName = par.getParameter("handler", null);
        if ( handlerName == null )
            throw new ProcessingException("sunRise LoginAction requires at least the handler parameter.");
        final Request request = ObjectModelHelper.getRequest( objectModel );

        // build authentication parameters
        SourceParameters authenticationParameters = new SourceParameters();
        String[] enum = par.getNames();
        if (enum != null) {
            for(int i = 0; i < enum.length; i++) {
                final String key = enum[i];
                if ( key.startsWith("parameter_") ) {
                    String value = par.getParameter(key);
                    if ( value.startsWith("request:") ) {
                        value = request.getParameter( value.substring("request:".length()) );
                        if (value == null) value = "";
                    }
                    authenticationParameters.setParameter( key.substring("parameter_".length()),
                                                           value);
                }
            }

        }

        Map map = null;

        // authenticate
        SunRise sunRise = null;
        try {
            sunRise = (SunRise) this.manager.lookup(SunRise.ROLE);
            Object o;

            o = sunRise.authenticate( handlerName,
                                      authenticationParameters);
            if ( null == o) {
                // success
                map = sunRise.createMap();

            } else {
                request.setAttribute(Constants.REQUEST_ATTRIBUTE_FAILED_LOGIN + handlerName, o);
            }
        } finally {
            this.manager.release(sunRise);
        }

        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("END act map="+map);
        }

        return map;
    }

}
